using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PlexityHide.GTP;

namespace LetsBuild_ProjectPlanner
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }


        private void Form1_Load(object sender, EventArgs e)
        {

            // Add calculated fields

            //first we derive max and min work for one task to maxWorkStop and minWorkStart
            DataColumn ddc1 = new DataColumn();
            ddc1.ColumnName = "maxWorkStop";
            ddc1.DataType = typeof(System.DateTime);
            ddc1.Expression = "max(child(workitems).Stop)";

            DataColumn ddc2 = new DataColumn();
            ddc2.ColumnName = "minWorkStart";
            ddc2.DataType = typeof(System.DateTime);
            ddc2.Expression = "min(child(workitems).Start)";

            //Then aggregate that value to a max and min for all subtasks in maxTaskStop and minTaskStart
            DataColumn ddc3 = new DataColumn();
            ddc3.ColumnName = "maxTaskStop";
            ddc3.DataType = typeof(System.DateTime);
            ddc3.Expression = "max(Child(subtasks).maxWorkStop)";

            DataColumn ddc4 = new DataColumn();
            ddc4.ColumnName = "minTaskStart";
            ddc4.DataType = typeof(System.DateTime);
            ddc4.Expression = "min(Child(subtasks).minWorkStart)";

            dataSet1.Tables["Task"].Columns.Add(ddc1);
            dataSet1.Tables["Task"].Columns.Add(ddc2);
            dataSet1.Tables["Task"].Columns.Add(ddc3);
            dataSet1.Tables["Task"].Columns.Add(ddc4);

            // Filter out true root nodes (those without owners)
            DataView dv = new DataView(dataSet1.Tables["Task"]);
            dv.RowFilter = "owningTaskId is null";
            gantt1.GridProperties.RootNodes_DataSource = dv;

            // Set up databind for timeItemLinks (relations between work items)
            gantt1.TimeItemLinks.NameInDS_StartKey = "WorkOwnerId";
            gantt1.TimeItemLinks.NameInDS_TargetKey = "WorkTargetId";
            gantt1.TimeItemLinks.DataSource = dataSet1.Tables["WorkRelation"];


            // Do Some time item styles based on the Work time item style
            gantt1.TimeItemLayouts.GetFromName("Work").CloneAndAddWithNewName("Green").Color = Color.Green;
            gantt1.TimeItemLayouts.GetFromName("Work").CloneAndAddWithNewName("Red").Color = Color.Red;
            gantt1.TimeItemLayouts.GetFromName("Work").CloneAndAddWithNewName("Blue").Color = Color.Blue;
            TimeItemLayout til = gantt1.TimeItemLayouts.GetFromName("Work").CloneAndAddWithNewName("Crazy");
            til.TimeItemStyle = TimeItemStyle.User;

            // Check if there is a xml file named project if so load it.
            if (System.IO.File.Exists("./project.xml"))
                dataSet1.ReadXml("./project.xml");

        }


        /// <summary>
        /// Based on context enable of disable menuitems
        /// </summary>
        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
        {
            for (int i = 0; i < contextMenuStrip1.Items.Count; i++)
                contextMenuStrip1.Items[i].Enabled = false;

            if (contextMenuStrip1.SourceControl == gantt1.Grid)
            {
                newTaskCollectionToolStripMenuItem.Enabled = true;
                newTaskToolStripMenuItem.Enabled = gantt1.Grid.FocusedCell != null;
                deleteToolStripMenuItem.Enabled = gantt1.Grid.FocusedCell != null;
            }
            else
                if (contextMenuStrip1.SourceControl == gantt1.TimeItemArea)
                {
                    if (gantt1.GanttRowFromPoint(gantt1.MouseDownPoint) != null)
                    {
                        // if isCollection then we cannot add work items
                        bool isCollection = (bool)(gantt1.GanttRowFromPoint(gantt1.MouseDownPoint).GridNode.ListItemWhenDataBound() as DataRowView)["IsCollection"];
                        newMilestoneToolStripMenuItem.Enabled = !isCollection;
                        newWorkItemToolStripMenuItem.Enabled = !isCollection;
                        newTaskRelationToolStripMenuItem.Enabled = !isCollection;
                        deleteToolStripMenuItem.Enabled = !isCollection && (gantt1.FocusedTimeItem != null);
                    }
                }
        }

        private void newTaskCollectionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AddTask(true, -1);
        }

        /// <summary>
        /// Add a row to the Task table
        /// </summary>
        private void AddTask(bool IsCollection, int owningTaskId)
        {
            DataRow r = dataSet1.Tables["Task"].NewRow();
            if (owningTaskId != -1)
            {
                r["OwningTaskId"] = owningTaskId;
            }
            r["Description"] = "<enter description>";
            r["IsCollection"] = IsCollection;

            dataSet1.Tables["Task"].Rows.Add(r);
        }

        private void newTaskToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (gantt1.Grid.FocusedCell != null)
            {
                int owningTaskId;
                if ((bool)(gantt1.Grid.FocusedCell.GridNode.ListItemWhenDataBound() as DataRowView)["IsCollection"])
                    owningTaskId = (int)(gantt1.Grid.FocusedCell.GridNode.ListItemWhenDataBound() as DataRowView)["id"];
                else
                    owningTaskId = (int)(gantt1.Grid.FocusedCell.GridNode.ParentNode.ListItemWhenDataBound() as DataRowView)["id"];
                AddTask(false, owningTaskId);
            }

        }


        /// <summary>
        /// For each row in the grid we initiate the databind for time items
        /// </summary>
        private void gantt1_OnEachListItemWhenDataBound_GridNode(object GTPComponent, PlexityHide.GTP.EachListItemWhenDataBoundArgs e)
        {
            // Avoid doing initial setup on every change
            if ((e.GTPObject as GridNode).SubNodes_DataSource == null)
            {
                // Set up databindings for subnodes
                DataView dv = (e.CurrencyManagerListItem as DataRowView).CreateChildView("SubTasks");
                (e.GTPObject as GridNode).SubNodes_DataSource = dv;

                if ((bool)(e.CurrencyManagerListItem as DataRowView)["IsCollection"])
                {
                    (e.GTPObject as GridNode).SetCellLayoutOnAll(gantt1.Grid.CellLayouts.GetFromName("Collection"));

                    // Set up databind for the span that is calculated by expression fields called minTaskStart and maxTaskStop
                    DataView dv2 = new DataView(dataSet1.Tables["Task"]);
                    dv2.RowFilter = "id=" + (int)(e.CurrencyManagerListItem as DataRowView)["id"];
                    Layer l = GanttRow.FromGridNode((e.GTPObject as GridNode)).Layers[0];
                    l.NameInDS_Start = "minTaskStart";
                    l.NameInDS_Stop = "maxTaskStop";
                    l.NameInDS_Identity = "id";
                    l.DataSource = dv2;

                }
                else
                {
                    (e.GTPObject as GridNode).SetCellLayoutOnAll(gantt1.Grid.CellLayouts.GetFromName("Normal"));

                    // Set up databind for work items
                    DataView dv2 = (e.CurrencyManagerListItem as DataRowView).CreateChildView("WorkItems");
                    Layer l = GanttRow.FromGridNode((e.GTPObject as GridNode)).Layers[0];
                    l.NameInDS_Start = "Start";
                    l.NameInDS_Stop = "Stop";
                    l.NameInDS_Identity = "id";
                    l.DataSource = dv2;
                }
            }
        }

        private void newWorkItemToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CreateWorkItem(false);
        }

        /// <summary>
        /// Add a row to the Work table
        /// </summary>
        private void CreateWorkItem(bool isMileStone)
        {
            GanttRow gr = gantt1.GanttRowFromPoint(gantt1.MouseDownPoint);
            if (gr != null)
            {
                int owningTaskID = (int)(gr.GridNode.ListItemWhenDataBound() as DataRowView)["Id"];

                DataRow r = dataSet1.Tables["Work"].NewRow();
                r["OwningTaskID"] = owningTaskID;
                r["Start"] = gantt1.DateScaler.PixelToTime(gantt1.MouseDownPoint.X);
                if (!isMileStone)
                    r["Stop"] = ((DateTime)r["Start"]).AddDays(1);
                r["IsMileStone"] = isMileStone;
                r["ProgressPercent"] = 0;
                r["Color"] = "Green";

                dataSet1.Tables["Work"].Rows.Add(r);

            }
        }

        private void newMilestoneToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CreateWorkItem(true);
        }



        /// <summary>
        /// Set up layout an color of available time items, based on if the represent work, milestone or span
        /// </summary>
        private void gantt1_OnEachListItemWhenDataBound_TimeItem(object GTPComponent, EachListItemWhenDataBoundArgs e)
        {
            if ((e.CurrencyManagerListItem as DataRowView).DataView.Table.TableName.ToLower() == "work")
            {
                if ((bool)(e.CurrencyManagerListItem as DataRowView)["IsMileStone"])
                {
                    (e.GTPObject as TimeItem).TimeItemLayout = gantt1.TimeItemLayouts.GetFromName("Milestone");
                }
                else
                {
                    (e.GTPObject as TimeItem).TimeItemLayout = gantt1.TimeItemLayouts.GetFromName((string)(e.CurrencyManagerListItem as DataRowView)["Color"]);
                }
            }
            else
            {
                // This is the collection time item, derived from the subtask's work items
                (e.GTPObject as TimeItem).TimeItemLayout = gantt1.TimeItemLayouts.GetFromName("Collection");

            }
        }

        private void newTaskRelationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            gantt1.EnterLinkCreateMode(true);
        }

        private void gantt1_OnEachListItemWhenDataBound_TimeItemLink(object GTPComponent, EachListItemWhenDataBoundArgs e)
        {
            (e.GTPObject as TimeItemLink).TimeItemLinkDrawStyle = TimeItemLinkDrawStyle.ZStyle;
            (e.GTPObject as TimeItemLink).Color = Color.BlueViolet;
        }

        /// <summary>
        /// Add a row or change the row in the WorkRelation table
        /// </summary>
        private void gantt1_OnTimeItem_LinkCreate(Gantt aGantt, TimeItemEventArgs e)
        {
            e.Allow = false; // We are databound so we just take the values and make a row of them
            int startid = (int)(e.TimeItem.ListItemWhenDataBound() as DataRowView)["id"];
            int targetid = (int)(e.LinkTargetTimeItem.ListItemWhenDataBound() as DataRowView)["id"];


            if (e.LinkUnderEdit == null)
            {
                // new link created
                DataRow r = dataSet1.Tables["WorkRelation"].NewRow();
                r["WorkOwnerID"] = startid;
                r["WorkTargetID"] = targetid;

                dataSet1.Tables["WorkRelation"].Rows.Add(r);
            }
            else
            {
                // existing link changed(e.LinkUnderEdit) 
                DataRow r = (e.LinkUnderEdit.ListItemWhenDataBound() as DataRowView).Row;
                r["WorkOwnerID"] = startid;
                r["WorkTargetID"] = targetid;
            }
            // ok done, go back to normal mode

            gantt1.EnterLinkCreateMode(false);


        }

        /// <summary>
        /// Handle move of task item so that the sub work items get moved along
        /// </summary>
        private void gantt1_OnTimeItem_Move(Gantt aGantt, TimeItemEventArgs e)
        {
            if ((e.TimeItem.ListItemWhenDataBound() as DataRowView).Row.Table.TableName.ToLower() == "task")
            {
                // This is the span that is calculated from subtask's work, 
                // Since it is calculated we cannot change it this way...
                e.Allow = false;
                // But we can interpret it as an attempt to move all subtask's work the e.Diff amount...
                for (int i = 0; i < e.TimeItem.GanttRow.GridNode.SubNodes.Count; i++)
                {
                    GanttRow gr = Gantt.GanttRowFromGridNode(e.TimeItem.GanttRow.GridNode.SubNodes[i]);
                    for (int ii = 0; ii < gr.Layers[0].Count; ii++)
                    {
                        gr.Layers[0][ii].Start += e.Diff;
                        try
                        {
                            gr.Layers[0][ii].Stop += e.Diff; // The milestone does not have a stop date so when we update it it may give an illegal date...
                        }
                        catch
                        {
                        }
                    }
                }

            }
        }

        /// <summary>
        /// Bring up databound properties dialog on dbl-click
        /// </summary>
        private void gantt1_OnTimeItemAreaMouseDown(object sender, MouseEventArgs e)
        {
            if (e.Clicks == 2)
            {
                TimeItem ti = gantt1.TimeItemFromPoint(e.Location);
                if (ti != null && (ti.ListItemWhenDataBound() as DataRowView).DataView.Table.TableName.ToLower() == "work")
                {

                    gantt1.MouseMoveCancel(); // dbl-Click can start move if not exact
                    Form_Work f = new Form_Work();
                    f.ViewTimeItem(ti);
                    f.ShowDialog();
                    f.Dispose();

                }

            }
        }

        /// <summary>
        /// User draw example, you can do anything... We also add tge progress bar inside work items
        /// </summary>
        private void gantt1_OnTimeItem_UserDraw(Gantt aGantt, TimeItemEventArgs e)
        {
            Console.WriteLine("Gantt1_OnTimeItem_UserDraw() fired for e.TimeItem.Identity: {0}", e.TimeItem.Identity);

            if (e.TimeItem.TimeItemLayout.Name == "Crazy")
            {
                // Ok,lets go crazy then...
                Point p = e.Rect.Location;
                int h = e.Rect.Height;
                Point p2 = p, p3 = p, p4 = p, p5 = p, p6 = p;
                p2.Offset(5, h);
                p3.Offset(10, 0);
                p4.Offset(15, h);
                p5.Offset(20, 0);
                p6.Offset(e.Rect.Width, h);

                Point[] points = { p, p2, p3, p4, p5, p6, p4 };
                e.G.DrawCurve(new Pen(Color.Tomato, 5), points);
                e.G.DrawCurve(new Pen(Color.Yellow, 3), points);
            }


            if (e.TimeItem.TimeItemLayout.ExtraAddOnDraw)
            {
                // Lets pull the Progress info from our data and show it inside the time item
                int progress = (int)(e.TimeItem.ListItemWhenDataBound() as DataRowView)["ProgressPercent"];
                int pBarWidth = (int)Math.Round(e.Rect.Width * progress / 100.0);
                int pBarPos = (int)(e.Rect.Top + e.Rect.Height / 2);
                e.G.FillRectangle(new SolidBrush(Color.Black), Rectangle.FromLTRB(e.Rect.Left, pBarPos - 2, e.Rect.Left + pBarWidth, pBarPos + 2));
            }
        }

        private void buttonSaveToFile_Click(object sender, EventArgs e)
        {
            dataSet1.WriteXml("./project.xml", XmlWriteMode.IgnoreSchema);
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.plexityhide.com/letsbuild_projectplanner.htm");
        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (contextMenuStrip1.SourceControl == gantt1.Grid)
            {
                if (gantt1.Grid.FocusedCell != null)
                {
                    DataRowView drv = gantt1.Grid.FocusedCell.GridNode.ListItemWhenDataBound() as DataRowView;
                    drv.BeginEdit();
                    drv.Delete();
                    drv.EndEdit();
                }

            }
            else
                if (contextMenuStrip1.SourceControl == gantt1.TimeItemArea)
                {
                    if (gantt1.GanttRowFromPoint(gantt1.MouseDownPoint) != null)
                    {
                        // if isCollection then we cannot add work items
                        if (gantt1.FocusedTimeItem != null)
                        {
                            bool isCollection = (bool)(gantt1.FocusedTimeItem.GanttRow.GridNode.ListItemWhenDataBound() as DataRowView)["IsCollection"];
                            if (!isCollection)
                            {
                                DataRowView drv = gantt1.FocusedTimeItem.ListItemWhenDataBound() as DataRowView;
                                drv.BeginEdit();
                                drv.Delete();
                                drv.EndEdit();
                            }
                        }
                    }
                }
        }

        private void buttonLoad_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                dataSet1.Clear();
                dataSet1.ReadXml(openFileDialog1.FileName, XmlReadMode.IgnoreSchema);
            }

        }
    }
}