using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PlexityHide.GTP;

namespace LetsBuild_ResourceBooker_wf
{
  public partial class Form1 : Form
  {
    public Form1()
    {
      InitializeComponent();
    }

    /// <summary>
    /// Enable the menu items depending on context
    /// </summary>
    private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
    {
      for (int i=0;i<contextMenuStrip1.Items.Count;i++)
        contextMenuStrip1.Items[i].Enabled=false;
        
      if (contextMenuStrip1.SourceControl==gantt1.Grid)
      {
        addNewBookableResourceToolStripMenuItem.Enabled=true;
        deleteBookableResourceToolStripMenuItem.Enabled = gantt1.Grid.GridStructure.FocusedCell!=null;      
      }
      else
      if (contextMenuStrip1.SourceControl == gantt1.TimeItemArea)
      {
        createABookingToolStripMenuItem.Enabled = gantt1.GanttRowFromPoint(gantt1.MouseDownPoint)!=null;
        removeBookingToolStripMenuItem.Enabled = gantt1.FocusedTimeItem != null;
      }

      
      
    }

    private void Form1_Load(object sender, EventArgs e)
    {
      DoAllTheInitWork();
    }
    public void DoAllTheInitWork()
    {
      gantt1.DateScaler.StartTime = new DateTime(2007, 2, 26).AddHours(5);
      gantt1.DateScaler.StopTime = gantt1.DateScaler.StartTime.AddHours(5);

      // Check if there is a xml file named project if so load it.
      if (System.IO.File.Exists("./resourcedata.xml"))
        dataSet1.ReadXml("./resourcedata.xml");
    }

    /// <summary>
    /// Create a new row in the Booking table
    /// </summary>
    private void addNewBookableResourceToolStripMenuItem_Click(object sender, EventArgs e)
    {
      DataRow r=dataSet1.Tables["Resource"].NewRow();
      dataSet1.Tables["Resource"].Rows.Add(r);
      r["Name"]="Room "+r["id"].ToString();
    }

    /// <summary>
    /// Create a new row in the Resource table and find the primary key for the resource
    /// </summary>
    private void createABookingToolStripMenuItem_Click(object sender, EventArgs e)
    {
      GanttRow gr=gantt1.GanttRowFromPoint(gantt1.MouseDownPoint);
      if (gr!=null)
      {
        int foreignKeyForResource=(int)(gr.GridNode.ListItemWhenDataBound() as DataRowView)["Id"];

        DataRow r = dataSet1.Tables["Booking"].NewRow();
        r["ResourceFK"] = foreignKeyForResource;
        r["Text"] = "Note";
        r["Booker"] = "<your name here>";
        r["Start"]  = gantt1.DateScaler.PixelToTime(gantt1.MouseDownPoint.Y);
        r["Stop"]=((DateTime)r["Start"]).AddHours(1);

        dataSet1.Tables["Booking"].Rows.Add(r);
      }
    }


    /// <summary>
    /// Our resources are represented by GridNodes (Column heads in the Schedule)
    /// In this event we set up databind for the bookings of each resource
    /// </summary>
    private void gantt1_OnEachListItemWhenDataBound_GridNode(object GTPComponent, EachListItemWhenDataBoundArgs e)
    {
      Layer l = Gantt.GanttRowFromGridNode(e.GTPObject as GridNode).Layers[0];
      if (l.DataSource==null)
      {
        // For each added node we must hook up the Bookings...
        DataView dv = (e.CurrencyManagerListItem as DataRowView).CreateChildView("Bookings");

        l.NameInDS_Identity = "Id";
        l.NameInDS_Start = "Start";
        l.NameInDS_Stop = "Stop";
        l.DataSource = dv;
      }
    }

    /// <summary>
    /// Our bookings are represented by TimeItems in the Schedule
    /// Here we extract further information on TimeItemTexts that we want to show
    /// </summary>
    private void gantt1_OnEachListItemWhenDataBound_TimeItem(object GTPComponent, EachListItemWhenDataBoundArgs e)
    {
      if ((e.GTPObject as TimeItem).TimeItemTexts.Count==0)
      {
        TimeItemText tit=new TimeItemText();
        tit.Text = (string) (e.CurrencyManagerListItem as DataRowView)["Text"];  
        tit.TimeItemTextLayout=gantt1.TimeItemTextLayouts.GetFromName("default");
        (e.GTPObject as TimeItem).TimeItemTexts.Add(tit);
        (e.GTPObject as TimeItem).TimeItemLayout=gantt1.TimeItemLayouts.GetFromName("default");

        tit=new TimeItemText();
        tit.Text = "Booking for "+(string)((e.GTPObject as TimeItem).GanttRow.GridNode.ListItemWhenDataBound() as DataRowView)["Name"];  
        tit.TimeItemTextLayout=gantt1.TimeItemTextLayouts.GetFromName("lower");
        (e.GTPObject as TimeItem).TimeItemTexts.Add(tit);
        (e.GTPObject as TimeItem).TimeItemLayout=gantt1.TimeItemLayouts.GetFromName("default");
      }      
      
    }

    /// <summary>
    /// A change of column in the scaler means that we should re-assign the resource 
    /// We do this by setting another foreign key on the row representing the booking
    /// </summary>
    private void gantt1_OnTimeItem_ChangeRow(Gantt aGantt, TimeItemEventArgs e)
    {
      // If we want to allow row changes when we are databound we must understand that a 
      // row change is only a switch of the foreign key for the TimeItem...
      e.Allow=false; // No gui operation, its a data operation
      if (e.TimeItem.ListItemWhenDataBound() is DataRowView)
      {
        (e.TimeItem.ListItemWhenDataBound() as DataRowView).BeginEdit();
        (e.TimeItem.ListItemWhenDataBound() as DataRowView)["ResourceFK"] = (int)(e.NewGanttRow.GridNode.ListItemWhenDataBound() as DataRowView)["id"];
        (e.TimeItem.ListItemWhenDataBound() as DataRowView).EndEdit();
      }
    }

    /// <summary>
    /// Delete the row from the table
    /// </summary>
    private void deleteBookableResourceToolStripMenuItem_Click(object sender, EventArgs e)
    {
      if (gantt1.Grid.GridStructure.FocusedCell!=null)
      {
        (gantt1.Grid.GridStructure.FocusedCell.Node.ListItemWhenDataBound() as DataRowView).Delete();
      }
    }

    /// <summary>
    /// Delete the row from the table
    /// </summary>
    private void removeBookingToolStripMenuItem_Click(object sender, EventArgs e)
    {
      if (gantt1.FocusedTimeItem != null)
      {
        (gantt1.FocusedTimeItem.ListItemWhenDataBound() as DataRowView).Delete();
      }

    }

    private void button1_Click(object sender, EventArgs e)
    {
      dataSet1.WriteXml("./resourcedata.xml", XmlWriteMode.IgnoreSchema);

    }
  }
}